/* LCD-Ansteuerung fr das Rx-Backend-Board (Freescale Kinetis MK22FN128VLH10) */

#include "StdTypes.h"
#include "LPC11E12.h"
#include "config.h"
#include "gdisplay.h"
#include "gdi.h"

/* Display: TG12864H, 128x64, Displaycontroller ST7567,
   Differenz ST7567 zu ST7565:
   nur interne LCD-Spannungsversorgung mglich,
   Booster MUSS benutzt werden. Default = 4x

   nur serieller Anschlu
   A0   auf PTC4
   /CS  auf PTD0
   SCL  auf PTD1
   DATA auf PTD2

   hier gendert auf SiFu-LP mit LPC11E12
 */

__inline void A0_low    (void) { P1_13 = 0; }
__inline void A0_high   (void) { P1_13 = 1; }

__inline void CS_low    (void) { P0_14 = 0; }
__inline void CS_high   (void) { P0_14 = 1; }

__inline void SCLK_low  (void) { P0_16 = 0; }
__inline void SCLK_high (void) { P0_16 = 1; }

__inline void DATA_low  (void) { P0_17 = 0; }
__inline void DATA_high (void) { P0_17 = 1; }


void gdelay (long i) { while (--i); }

byte LCD_Out (byte b)
{ byte m;
  m = 128;
  CS_low();
  while (m)
  { SCLK_low();
    if (b & m)
      DATA_high();
    else
      DATA_low();
    m = (m>>1) & 127;
    gdelay(6);
    SCLK_high();
    gdelay(5);
  }
  CS_high();
  return b;
}


byte  BWSP[BWSP_Len];     // der Display-RAM
bool  LcdInvalid;
byte  LcdKontrast;



/* Display-Kommandos A0=low*/
#define display_off   0xAE
#define display_on    0xAF

#define set_startline 0x40     // line+(0..63)
#define set_page      0xB0     // page+(0..8)

#define col_low       0x10     // col_low+(column & 15)
#define col_hi        0        // col_hi+(column>>4)   (column = 0..131)

#define seg_normal    0xA0     // Segment-Treiber normal
#define seg_revers    0xA1     // Segment-Treiber invertiert (R<-->L)

#define disp_normal   0xA6     // Pixel normal
#define disp_invert   0xA7     // Pixel invertiert

#define allpix_norm   0xA4     // Pixel normal
#define allpix_on     0xA5     // alle Pixel ein

#define setbias_0     0xA2     // 1/9
#define setbias_1     0xA3     // 1/7

#define rmw_start     0xE0     // Read-Modify_Write starten
#define rmw_end       0xEE     // Read-Modify_Write ende

#define reset_init    0xE2     // line,col,page,volts

#define com_normal    0xC0     // COM-Treiber normal
#define com_revers    0xC8     // COM-Treiber invertiert (U<-->D)

#define power         0x28     // power+(boost,reg,amp)
#define boost  4               // Booster ein
#define reg    2               // Regulator ein
#define amp    1               // Spannungsfolger ein

#define regul_ratio   0x20     // regul_ratio+(0..7) R/R fr V0 Regulator
#define set_ev        0x81     // set_ev, danach 1 Byte Kontrastregister-Inhalt 1..63
#define set_booster   0xF8     // set_booster, danach 1=5x, 0= 4x


/*
merke: Sleep-Mode (=Standby) erreicht man per
- display_off
- all_on
und Verlassen in umgekehrter Richtung:
- all_normal
- display_on
*/
void LCD_DoSleep (void)
{ A0_low();
  LCD_Out(display_off);
  gdelay(10);
  LCD_Out(allpix_on);
}

void LCD_ExitSleep (void)
{ A0_low();
  LCD_Out(allpix_norm);
  gdelay(10);
  LCD_Out(display_on);
}


void LCD_Init (void)
{ CS_low();
  A0_low();
  LCD_Out(power | boost);
  gdelay(100);
  LCD_Out(power | boost | reg);
  gdelay(100);
  LCD_Out(power | boost | reg | amp);
  gdelay(100);

  LCD_Out(set_booster); LCD_Out(0);       // 4x
  LCD_Out(regul_ratio+4);
  LCD_Out(set_ev);
  LCD_Out(43);
  LcdKontrast = 32;

  LCD_Out(set_startline+0);
  LCD_Out(allpix_norm);
  LCD_Out(display_on);

  LCD_Out(seg_normal);
  LCD_Out(com_revers);

  gedreht = false;
}

void LCD_Blocktransfer (void)
{ int y, pg, idx;

  pg = 0;
  idx = 0;
  while (pg<8)
  { A0_low();
    LCD_Out(set_page+pg);
    LCD_Out(col_low+0);
    LCD_Out(col_hi+0);
    A0_high();
    y = 0;
    while (y<128)
    { LCD_Out(BWSP[idx++]);
      ++y;
    }
    ++pg;
  }
  LcdInvalid = false;
}

// Display ablschen
void LCD_Clear (byte b)
{ int i;
  i = 0;
  while (i<BWSP_Len)
  { BWSP[i] = b; ++i; }
}

void LCD_SetKontrast (byte kontrast)
{ if (kontrast>63) kontrast = 63;
  A0_low();
  LCD_Out(set_ev);
  LcdKontrast = kontrast;
  LCD_Out(kontrast);     // Vlcd setzen
  A0_high();
}

    /* ende */
